@@ -20,17 +20,13 @@ import logging
 import sys
 import traceback
 
-from oslo.config import cfg
-from oslo import messaging
 import six
 
+from oslo import messaging
 from oslo.messaging import _utils as utils
-from oslo.messaging.openstack.common import importutils
+from oslo.messaging.openstack.common.gettextutils import _
 from oslo.messaging.openstack.common import jsonutils
 
-# FIXME(markmc): remove this
-_ = lambda s: s
-
 LOG = logging.getLogger(__name__)
 
 _EXCEPTIONS_MODULE = 'exceptions' if six.PY2 else 'builtins'
@@ -73,18 +69,6 @@ _MESSAGE_KEY = 'oslo.message'
 
 _REMOTE_POSTFIX = '_Remote'
 
-_exception_opts = [
-    cfg.ListOpt('allowed_rpc_exception_modules',
-                default=['oslo.messaging.exceptions',
-                         'nova.exception',
-                         'cinder.exception',
-                         _EXCEPTIONS_MODULE,
-                         ],
-                help='Modules of exceptions that are permitted to be '
-                     'recreated upon receiving exception data from an rpc '
-                     'call.'),
-]
-
 
 class RPCException(Exception):
     msg_fmt = _("An unknown RPC related exception occurred.")
@@ -101,33 +85,13 @@ class RPCException(Exception):
                 # log the issue and the kwargs
                 LOG.exception(_('Exception in string format operation'))
                 for name, value in six.iteritems(kwargs):
-                    LOG.error("%s: %s" % (name, value))
+                    LOG.error("%s: %s", name, value)
                 # at least get the core message out if something happened
                 message = self.msg_fmt
 
         super(RPCException, self).__init__(message)
 
 
-class RemoteError(RPCException):
-    """Signifies that a remote class has raised an exception.
-
-    Contains a string representation of the type of the original exception,
-    the value of the original exception, and the traceback.  These are
-    sent to the parent as a joined string so printing the exception
-    contains all of the relevant info.
-
-    """
-    msg_fmt = _("Remote error: %(exc_type)s %(value)s\n%(traceback)s.")
-
-    def __init__(self, exc_type=None, value=None, traceback=None):
-        self.exc_type = exc_type
-        self.value = value
-        self.traceback = traceback
-        super(RemoteError, self).__init__(exc_type=exc_type,
-                                          value=value,
-                                          traceback=traceback)
-
-
 class Timeout(RPCException):
     """Signifies that a timeout has occurred.
 
@@ -194,83 +158,6 @@ class Connection(object):
         """
         raise NotImplementedError()
 
-    def create_consumer(self, topic, proxy, fanout=False):
-        """Create a consumer on this connection.
-
-        A consumer is associated with a message queue on the backend message
-        bus.  The consumer will read messages from the queue, unpack them, and
-        dispatch them to the proxy object.  The contents of the message pulled
-        off of the queue will determine which method gets called on the proxy
-        object.
-
-        :param topic: This is a name associated with what to consume from.
-                      Multiple instances of a service may consume from the same
-                      topic. For example, all instances of nova-compute consume
-                      from a queue called "compute".  In that case, the
-                      messages will get distributed amongst the consumers in a
-                      round-robin fashion if fanout=False.  If fanout=True,
-                      every consumer associated with this topic will get a
-                      copy of every message.
-        :param proxy: The object that will handle all incoming messages.
-        :param fanout: Whether or not this is a fanout topic.  See the
-                       documentation for the topic parameter for some
-                       additional comments on this.
-        """
-        raise NotImplementedError()
-
-    def create_worker(self, topic, proxy, pool_name):
-        """Create a worker on this connection.
-
-        A worker is like a regular consumer of messages directed to a
-        topic, except that it is part of a set of such consumers (the
-        "pool") which may run in parallel. Every pool of workers will
-        receive a given message, but only one worker in the pool will
-        be asked to process it. Load is distributed across the members
-        of the pool in round-robin fashion.
-
-        :param topic: This is a name associated with what to consume from.
-                      Multiple instances of a service may consume from the same
-                      topic.
-        :param proxy: The object that will handle all incoming messages.
-        :param pool_name: String containing the name of the pool of workers
-        """
-        raise NotImplementedError()
-
-    def join_consumer_pool(self, callback, pool_name, topic, exchange_name):
-        """Register as a member of a group of consumers.
-
-        Uses given topic from the specified exchange.
-        Exactly one member of a given pool will receive each message.
-
-        A message will be delivered to multiple pools, if more than
-        one is created.
-
-        :param callback: Callable to be invoked for each message.
-        :type callback: callable accepting one argument
-        :param pool_name: The name of the consumer pool.
-        :type pool_name: str
-        :param topic: The routing topic for desired messages.
-        :type topic: str
-        :param exchange_name: The name of the message exchange where
-                              the client should attach. Defaults to
-                              the configured exchange.
-        :type exchange_name: str
-        """
-        raise NotImplementedError()
-
-    def consume_in_thread(self):
-        """Spawn a thread to handle incoming messages.
-
-        Spawn a thread that will be responsible for handling all incoming
-        messages for consumers that were set up on this connection.
-
-        Message dispatching inside of this is expected to be implemented in a
-        non-blocking manner.  An example implementation would be having this
-        thread pull messages in for all of the consumers, but utilize a thread
-        pool for dispatching the messages to the proxy objects.
-        """
-        raise NotImplementedError()
-
 
 def _safe_log(log_func, msg, msg_data):
     """Sanitizes the msg_data field before logging."""
@@ -309,8 +196,8 @@ def serialize_remote_exception(failure_info, log_failure=True):
 
     # NOTE(matiu): With cells, it's possible to re-raise remote, remote
     # exceptions. Lets turn it back into the original exception type.
-    cls_name = str(failure.__class__.__name__)
-    mod_name = str(failure.__class__.__module__)
+    cls_name = six.text_type(failure.__class__.__name__)
+    mod_name = six.text_type(failure.__class__.__module__)
     if (cls_name.endswith(_REMOTE_POSTFIX) and
             mod_name.endswith(_REMOTE_POSTFIX)):
         cls_name = cls_name[:-len(_REMOTE_POSTFIX)]
@@ -331,7 +218,7 @@ def serialize_remote_exception(failure_info, log_failure=True):
 
 
 def deserialize_remote_exception(data, allowed_remote_exmods):
-    failure = jsonutils.loads(str(data))
+    failure = jsonutils.loads(six.text_type(data))
 
     trace = failure.get('tb', [])
     message = failure.get('message', "") + "\n" + "\n".join(trace)
@@ -344,7 +231,8 @@ def deserialize_remote_exception(data, allowed_remote_exmods):
         return messaging.RemoteError(name, failure.get('message'), trace)
 
     try:
-        mod = importutils.import_module(module)
+        __import__(module)
+        mod = sys.modules[module]
         klass = getattr(mod, name)
         if not issubclass(klass, Exception):
             raise TypeError("Can only deserialize Exceptions")
@@ -393,31 +281,9 @@ class CommonRpcContext(object):
         return self.from_dict(self.to_dict())
 
     def update_store(self):
-        #local.store.context = self
+        # local.store.context = self
         pass
 
-    def elevated(self, read_deleted=None, overwrite=False):
-        """Return a version of this context with admin flag set."""
-        # TODO(russellb) This method is a bit of a nova-ism.  It makes
-        # some assumptions about the data in the request context sent
-        # across rpc, while the rest of this class does not.  We could get
-        # rid of this if we changed the nova code that uses this to
-        # convert the RpcContext back to its native RequestContext doing
-        # something like nova.context.RequestContext.from_dict(ctxt.to_dict())
-
-        context = self.deepcopy()
-        context.values['is_admin'] = True
-
-        context.values.setdefault('roles', [])
-
-        if 'admin' not in context.values['roles']:
-            context.values['roles'].append('admin')
-
-        if read_deleted is not None:
-            context.values['read_deleted'] = read_deleted
-
-        return context
-
 
 class ClientException(Exception):
     """Encapsulates actual exception expected to be hit by a RPC proxy object.
@@ -429,32 +295,6 @@ class ClientException(Exception):
         self._exc_info = sys.exc_info()
 
 
-def catch_client_exception(exceptions, func, *args, **kwargs):
-    try:
-        return func(*args, **kwargs)
-    except Exception as e:
-        if type(e) in exceptions:
-            raise ClientException()
-        else:
-            raise
-
-
-def client_exceptions(*exceptions):
-    """Decorator for manager methods that raise expected exceptions.
-
-    Marking a Manager method with this decorator allows the declaration
-    of expected exceptions that the RPC layer should not consider fatal,
-    and not log as if they were generated in a real error scenario. Note
-    that this will cause listed exceptions to be wrapped in a
-    ClientException, which is used internally by the RPC layer.
-    """
-    def outer(func):
-        def inner(*args, **kwargs):
-            return catch_client_exception(exceptions, func, *args, **kwargs)
-        return inner
-    return outer
-
-
 def serialize_msg(raw_msg):
     # NOTE(russellb) See the docstring for _RPC_ENVELOPE_VERSION for more
     # information about this format.
