@@ -21,11 +21,8 @@ import json
 import logging
 
 from oslo.config import cfg
-
 from oslo.messaging._drivers import matchmaker as mm
-
-# FIXME(markmc): remove this
-_ = lambda s: s
+from oslo.messaging.openstack.common.gettextutils import _
 
 matchmaker_opts = [
     # Matchmaker ring file
@@ -74,7 +71,7 @@ class RoundRobinRingExchange(RingExchange):
         if not self._ring_has(key):
             LOG.warn(
                 _("No key defining hosts for topic '%s', "
-                  "see ringfile") % (key, )
+                  "see ringfile"), key
             )
             return []
         host = next(self.ring0[key])
@@ -92,7 +89,7 @@ class FanoutRingExchange(RingExchange):
         if not self._ring_has(nkey):
             LOG.warn(
                 _("No key defining hosts for topic '%s', "
-                  "see ringfile") % (nkey, )
+                  "see ringfile"), nkey
             )
             return []
         return map(lambda x: (key + '.' + x, x), self.ring[nkey])
