@@ -75,7 +75,8 @@ class MessageHandlingServer(object):
         :type transport: Transport
         :param dispatcher: a callable which is invoked for each method
         :type dispatcher: callable
-        :param executor: name of message executor - e.g. 'eventlet', 'blocking'
+        :param executor: name of message executor - for example
+                         'eventlet', 'blocking'
         :type executor: str
         """
         self.conf = transport.conf
@@ -107,9 +108,7 @@ class MessageHandlingServer(object):
         process, thread or co-operatively scheduled coroutine or simply by
         registering a callback with an event loop. Similarly, the executor may
         choose to dispatch messages in a new thread, coroutine or simply the
-        current thread. An RPCServer subclass is available for each I/O
-        strategy supported by the library, so choose the subclass appropriate
-        for your program.
+        current thread.
         """
         if self._executor is not None:
             return
