@@ -15,11 +15,11 @@
 
 import fixtures
 from mox3 import mox
-from oslo.config import cfg
 import six
 from stevedore import driver
 import testscenarios
 
+from oslo.config import cfg
 from oslo import messaging
 from oslo.messaging import transport
 from tests import utils as test_utils
@@ -202,7 +202,7 @@ class GetTransportSadPathTestCase(test_utils.BaseTestCase):
 
             for k, v in self.ex.items():
                 self.assertTrue(hasattr(ex, k))
-                self.assertEqual(str(getattr(ex, k)), v)
+                self.assertEqual(v, str(getattr(ex, k)))
 
 
 # FIXME(markmc): this could be used elsewhere
@@ -270,7 +270,7 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
         self.mox.StubOutWithMock(t._driver, 'send')
         t._driver.send(self._target, 'ctxt', 'message',
                        wait_for_reply=None,
-                       timeout=None)
+                       timeout=None, retry=None)
         self.mox.ReplayAll()
 
         t._send(self._target, 'ctxt', 'message')
@@ -281,22 +281,34 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
         self.mox.StubOutWithMock(t._driver, 'send')
         t._driver.send(self._target, 'ctxt', 'message',
                        wait_for_reply='wait_for_reply',
-                       timeout='timeout')
+                       timeout='timeout', retry='retry')
         self.mox.ReplayAll()
 
         t._send(self._target, 'ctxt', 'message',
                 wait_for_reply='wait_for_reply',
-                timeout='timeout')
+                timeout='timeout', retry='retry')
 
     def test_send_notification(self):
         t = transport.Transport(_FakeDriver(cfg.CONF))
 
         self.mox.StubOutWithMock(t._driver, 'send_notification')
-        t._driver.send_notification(self._target, 'ctxt', 'message', 1.0)
+        t._driver.send_notification(self._target, 'ctxt', 'message', 1.0,
+                                    retry=None)
         self.mox.ReplayAll()
 
         t._send_notification(self._target, 'ctxt', 'message', version=1.0)
 
+    def test_send_notification_all_args(self):
+        t = transport.Transport(_FakeDriver(cfg.CONF))
+
+        self.mox.StubOutWithMock(t._driver, 'send_notification')
+        t._driver.send_notification(self._target, 'ctxt', 'message', 1.0,
+                                    retry=5)
+        self.mox.ReplayAll()
+
+        t._send_notification(self._target, 'ctxt', 'message', version=1.0,
+                             retry=5)
+
     def test_listen(self):
         t = transport.Transport(_FakeDriver(cfg.CONF))
 
@@ -305,3 +317,50 @@ class TestTransportMethodArgs(test_utils.BaseTestCase):
         self.mox.ReplayAll()
 
         t._listen(self._target)
+
+
+class TestTransportUrlCustomisation(test_utils.BaseTestCase):
+    def setUp(self):
+        super(TestTransportUrlCustomisation, self).setUp()
+        self.url1 = transport.TransportURL.parse(self.conf, "fake://vhost1")
+        self.url2 = transport.TransportURL.parse(self.conf, "fake://vhost2")
+        self.url3 = transport.TransportURL.parse(self.conf, "fake://vhost1")
+
+    def test_hash(self):
+        urls = {}
+        urls[self.url1] = self.url1
+        urls[self.url2] = self.url2
+        urls[self.url3] = self.url3
+        self.assertEqual(2, len(urls))
+
+    def test_eq(self):
+        self.assertEqual(self.url1, self.url3)
+        self.assertNotEqual(self.url1, self.url2)
+
+
+class TestTransportHostCustomisation(test_utils.BaseTestCase):
+    def setUp(self):
+        super(TestTransportHostCustomisation, self).setUp()
+        self.host1 = transport.TransportHost("host1", 5662, "user", "pass")
+        self.host2 = transport.TransportHost("host1", 5662, "user", "pass")
+        self.host3 = transport.TransportHost("host1", 5663, "user", "pass")
+        self.host4 = transport.TransportHost("host1", 5662, "user2", "pass")
+        self.host5 = transport.TransportHost("host1", 5662, "user", "pass2")
+        self.host6 = transport.TransportHost("host2", 5662, "user", "pass")
+
+    def test_hash(self):
+        hosts = {}
+        hosts[self.host1] = self.host1
+        hosts[self.host2] = self.host2
+        hosts[self.host3] = self.host3
+        hosts[self.host4] = self.host4
+        hosts[self.host5] = self.host5
+        hosts[self.host6] = self.host6
+        self.assertEqual(5, len(hosts))
+
+    def test_eq(self):
+        self.assertEqual(self.host1, self.host2)
+        self.assertNotEqual(self.host1, self.host3)
+        self.assertNotEqual(self.host1, self.host4)
+        self.assertNotEqual(self.host1, self.host5)
+        self.assertNotEqual(self.host1, self.host6)
